package eu.maiora.routes

import eu.maiora.model.*
import io.ktor.http.*
import io.ktor.server.application.*
import io.ktor.server.auth.*
import io.ktor.server.auth.jwt.*
import io.ktor.server.response.*
import io.ktor.server.routing.*


fun Route.disponibilitaArticoli(articoliCrossQRicambiRepository: ArticoliCrossQRicambiRepository,
                                viewEcommerceArticoliRepository: ViewEcommerceArticoliRepository,
                                viewEcommercePrezSconDispRepository: ViewEcommercePrezSconDispRepository,
                                viewClientiAgentiRepository : ViewClientiAgentiRepository,
                                viewEcommerceValoreUsatoRepository: ViewEcommerceValoreUsatoRepository){
    route("/api/disponibilitaArticoli"){
        authenticate("auth-jwt") {
            get("{codiceArticolo}"){
                val username = call.authentication.principal<JWTPrincipal>()?.payload?.subject ?: ""

                // Ottieni il codice articolo dal percorso
                val codiceArticolo = call.parameters["codiceArticolo"]

                if (codiceArticolo == null) {
                    call.respondText("Codice articolo non valido", status = HttpStatusCode.BadRequest)
                    return@get
                }

                //TODO verifica utente loggato: se  qricambi partner, richiesta generica,
                // altrimenti richiesta puntuale
                // al momento gestiamo solo richiesta generica per tutti

                //ricerca del codice articolo
                //- verifico che il codice esista nel sistema interno, ovvero nella tabella ARTICOLI_CROSS_QRICAMBI
                //  la ricerca va effettuata prima per CODICE_ARTICOLO e poi per CODICE_CROSS
                //- se non esiste, restituisco not found
                //- altrimenti, recupero l'id dell'articolo dalla vista VIEW_ECOMMERCE_ARTICOLI
                //  e tutti i dati da mostrare (vedi GET_PREZ_SCON_DISP, named query in ViewEcommercePrezSconDisp)

                if(username.equals("qricambi")){
                    val idClienteQRicambi = 10000023706
                    val listArticoliCrossQRicambi = articoliCrossQRicambiRepository.articoloByCodArt(codiceArticolo)
                    if(listArticoliCrossQRicambi.isNotEmpty()){
                        //call.respond(listArticoliCrossQRicambi)
                        val articoloEcommerce = viewEcommerceArticoliRepository.articoloByCodice(listArticoliCrossQRicambi[0].codiceArticolo)
                        if(articoloEcommerce != null){
                            //call.respond(articoloEcommerce)
                            val dataValoreUsato = viewEcommerceValoreUsatoRepository.getValoreByIdArtIdClie(idClienteQRicambi, articoloEcommerce.id)
                            val data = viewEcommercePrezSconDispRepository.disponibilitaByIdArtIdClie(idClienteQRicambi, articoloEcommerce.id)
                            if(data != null){
                                var listNumPezziPerMagazzino = ArrayList<NumPezziPerMagazzino>()
                                /*data.forEach { e ->
                                    listNumPezziPerMagazzino.add(NumPezziPerMagazzino(e.magazzino, e.numeroPezzi.toString()))
                                }*/
                                var numPezziDisponibili = 0;
                                data.forEach { e ->
                                    numPezziDisponibili +=  e.numeroPezzi
                                }
                                listNumPezziPerMagazzino.add(NumPezziPerMagazzino("", numPezziDisponibili.toString()))

                                val result = DisponibilitaArticoli(articoloEcommerce.codice,
                                    articoloEcommerce.descrizione,
                                    listArticoliCrossQRicambi[0].costruttore,
                                    data[0].netto.toString(),
                                    data[0].numeroPezzi > 0,
                                    listNumPezziPerMagazzino,
                                    (dataValoreUsato?.firstOrNull()?.valoreUsato ?: "0.0").toString()
                                )
                                call.respond(result)
                            }
                            else {
                                call.respondText("Articolo ecommerce non trovato (prezzi sconti e disponibilita)", status = HttpStatusCode.NotFound)
                            }

                        }
                        else {
                            call.respondText("Articolo ecommerce non trovato", status = HttpStatusCode.NotFound)
                        }
                    }
                    else {
                        call.respondText("Articolo non trovato", status = HttpStatusCode.NotFound)
                    }
                }
                else {
                    //verificare il cliente collegato all'agente/username
                    val cliente = viewClientiAgentiRepository.clienteByAgente(username)
                    if(cliente != null){
                        val listArticoliCrossQRicambi = articoliCrossQRicambiRepository.articoloByCodArt(codiceArticolo)
                        if(!listArticoliCrossQRicambi.isEmpty()){
                            //call.respond(listArticoliCrossQRicambi)
                            val articoloEcommerce = viewEcommerceArticoliRepository.articoloByCodice(listArticoliCrossQRicambi[0].codiceArticolo)
                            if(articoloEcommerce != null){
                                //call.respond(articoloEcommerce)
                                val dataValoreUsato = viewEcommerceValoreUsatoRepository.getValoreByIdArtIdClie(cliente.idCliente, articoloEcommerce.id)
                                val data = viewEcommercePrezSconDispRepository.disponibilitaByIdArtIdClie(cliente.idCliente, articoloEcommerce.id)
                                if(!data.isNullOrEmpty()){
                                    var listNumPezziPerMagazzino = ArrayList<NumPezziPerMagazzino>()
                                    data.forEach { e ->
                                        listNumPezziPerMagazzino.add(NumPezziPerMagazzino(e.magazzino, e.numeroPezzi.toString()))
                                    }
                                    //var numPezziDisponibili = 0;
                                    /*data.forEach { e ->
                                        numPezziDisponibili +=  e.numeroPezzi
                                    }*/
                                    //listNumPezziPerMagazzino.add(NumPezziPerMagazzino("", numPezziDisponibili.toString()))

                                    val result = DisponibilitaArticoli(articoloEcommerce.codice,
                                        articoloEcommerce.descrizione,
                                        listArticoliCrossQRicambi[0].costruttore,
                                        data[0].netto.toString(),
                                        data[0].numeroPezzi > 0,
                                        listNumPezziPerMagazzino,
                                        dataValoreUsato?.get(0)?.valoreUsato.toString()
                                    )
                                    call.respond(result)
                                }
                                else {
                                    call.respondText("Articolo ecommerce non trovato (prezzi sconti e disponibilita)", status = HttpStatusCode.NotFound)
                                }

                            }
                            else {
                                call.respondText("Articolo ecommerce non trovato", status = HttpStatusCode.NotFound)
                            }
                        }
                        else {
                            call.respondText("Articolo non trovato", status = HttpStatusCode.NotFound)
                        }
                    }
                    else {
                        call.respondText("Cliente collegato all'agente non trovato", status = HttpStatusCode.NotFound)
                    }
                }


                // Cerca la tessera per codice fiscale
                //val listaMovimenti = movimentiRepository.movimentiByIdTessera(idTessera.toLong())


                /*if (listaMovimenti != null) {
                    call.respond(listaMovimenti)
                } else {
                    call.respondText("Movimenti non trovati", status = HttpStatusCode.NotFound)
                }*/
            }
        }

    }
}